<?php

namespace Unific\Extension\Connection\Rest;

use Unific\Extension\Connection\ConnectionInterface;

class Connection extends \Unific\Extension\Connection\Connection implements ConnectionInterface
{
    protected $urlData = array();

    /**
     * Setup the initial connection
     * @return $this
     */
    public function setup()
    {
        parent::setup();

        return $this;
    }

    /**
     * @return mixed|\Unific\Extension\Connection\Connection
     */
    public function doRequest()
    {
        return parent::doRequest();
    }

    /**
     * @return mixed|\Unific\Extension\Connection\Connection
     */
    public function handleResponse()
    {
        return parent::handleResponse();
    }

    /**
     * @return mixed|string
     */
    public function getRestPath()
    {
        return (isset($this->urlData['query'])) ? $this->urlData['path'] . '?' . urlencode($this->urlData['query']) : $this->urlData['path'];
    }

    /**
     * @param $url
     * @param array $data
     * @param array $extraHeaders
     * @param bool $queue
     * @param $requestType
     * @param bool $historical
     * @return mixed
     */
    public function executeRequest($url, $data = array(), $extraHeaders = array(), $queue = false, $requestType = \Zend\Http\Request::METHOD_POST, $historical = false)
    {
        // Ensure the request is always sent to the queue if its in burst mode
        if($queue || $this->scopeConfig->getValue('unific/extension/mode') == 'burst')
        {
            return $this->queueHelper->queue($url, $data, $extraHeaders, $requestType, $historical);
        } else {
            return $this->sendData($url, $data, $extraHeaders, $requestType);
        }
    }

    /**
     * @param $url
     * @param array $data
     * @param array $extraHeaders
     * @param $requestType
     * @return \Zend\Http\Response
     */
    public function sendData($url, $data = array(), $extraHeaders = array(), $requestType = \Zend\Http\Request::METHOD_POST)
    {
        $extraHeaders['Content-Type'] = 'application/json';
        $extraHeaders['Accept'] = 'application/json';

        $extraHeaders['X-UNIFIC-INTEGRATION-ID'] = $this->scopeConfig->getValue('unific/extension/integration', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $extraHeaders['X-MAGENTO-UNIFIC-HMAC'] = $this->hmacHelper->generateHmac($data);

        $this->httpClient->setOptions(array('sslverifypeer' => null, 'sslallowselfsigned' => true));
        $this->httpClient->setAdapter(new \Zend\Http\Client\Adapter\Curl());
        $this->httpClient->setEncType(\Zend\Http\Client::ENC_FORMDATA);
        $this->httpClient->setUri($url);

        $httpHeaders = $this->httpHeadersFactory->create();
        $httpHeaders->addHeaders($extraHeaders);

        $httpRequest = $this->httpRequestFactory->create();
        $httpRequest->setUri($url);
        $httpRequest->setMethod($requestType);
        $httpRequest->setHeaders($httpHeaders);

        switch($requestType) {
            case \Zend\Http\Request::METHOD_POST:
                $httpRequest->setContent(json_encode($data));
                break;
            case \Zend\Http\Request::METHOD_PUT:
                $httpRequest->setContent(json_encode($data));
                break;
            default:
                $this->httpClient->setQuery($data);
                break;
        }

        return $this->httpClient->send($httpRequest);
    }


    /**
     * @param $url
     * @param array $data
     * @param array $extraHeaders
     * @param bool $queue
     * @return
     */
    public function post($url, $data = array(), $extraHeaders = array(), $queue = false)
    {
        return $this->executeRequest($url,  $data, $extraHeaders, $queue, \Zend\Http\Request::METHOD_POST);
    }

    /**
     * @param $url
     * @param array $data
     * @param array $extraHeaders
     * @param bool $queue
     * @return
     */
    public function get($url, $data = array(), $extraHeaders = array(), $queue = false)
    {
        return $this->executeRequest($url, $data, $extraHeaders, $queue, \Zend\Http\Request::METHOD_GET);
    }

    /**
     * @param $url
     * @param array $data
     * @param array $extraHeaders
     * @param bool $queue
     * @return
     */
    public function put($url, $data = array(), $extraHeaders = array(), $queue = false)
    {
        return $this->executeRequest($url, $data, $extraHeaders, $queue, \Zend\Http\Request::METHOD_PUT);
    }

    /**
     * @param $url
     * @param array $data
     * @param array $extraHeaders
     * @param bool $queue
     * @return
     */
    public function delete($url, $data = array(), $extraHeaders = array(), $queue = false)
    {
        return $this->executeRequest($url, $data, $extraHeaders, $queue, \Zend\Http\Request::METHOD_DELETE);
    }
}