<?php

namespace Unific\Extension\Helper\Data;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Cart extends AbstractData
{
    protected $customerSession;

    // Holds a Quote API DATA object
    private $cart;

    protected $returnData = array(
    );

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Magento\Customer\Model\Session $customerSession
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Magento\Customer\Model\Session $customerSession
    )
    {
        $this->customerSession = $customerSession;

        parent::__construct($context, $logger, $filterHelper);
    }

    /**
     * @param \Magento\Quote\Model\Quote $cart
     */
    public function setCart(\Magento\Quote\Model\Quote $cart)
    {
        $this->cart = $cart;
        $this->setCartInfo();
    }

    public function setAddressData(\Magento\Quote\Model\Quote\Address $address)
    {
        $this->returnData['customer_email'] = $address->getEmail();
        $this->returnData['customer_firstname'] = $address->getFirstname();
        $this->returnData['customer_middlename'] = $address->getMiddlename();
        $this->returnData['customer_lastname'] = $address->getLastname();
        $this->returnData['customer_prefix'] = $address->getPrefix();
        $this->returnData['customer_suffix'] = $address->getSuffix();
        $this->returnData['customer_postcode'] = $address->getPostcode();
        $this->returnData['customer_street'] = $address->getStreetFull();
        $this->returnData['customer_city'] = $address->getCity();
        $this->returnData['customer_telephone'] = $address->getTelephone();
        $this->returnData['customer_fax'] = $address->getFax();
        $this->returnData['customer_company'] = $address->getCompany();
        $this->returnData['customer_region'] = $address->getRegionCode();
        $this->returnData['customer_country'] = $address->getCountryModel()->getName();
    }

    /**
     * @return \Magento\Quote\Model\Data\QuoteInterface
     */
    public function getCart()
    {
        return $this->cart;
    }

    /**
     * @return mixed
     */
    protected function setCartInfo()
    {
        $this->returnData = $this->cart->getData();

        if($this->customerSession->isLoggedIn())
        {
            $this->returnData['customer_email'] = $this->customerSession->getCustomer()->getEmail();
            $this->returnData['customer_firstname'] = $this->customerSession->getCustomer()->getFirstname();
            $this->returnData['customer_middlename'] = $this->customerSession->getCustomer()->getMiddlename();
            $this->returnData['customer_lastname'] = $this->customerSession->getCustomer()->getLastname();
        }

        $this->returnData['items'] = array();
        foreach($this->cart->getAllItems() as $item)
        {
            $this->returnData['items'][] = $item->getData();
        }
    }

    /**
     * @return array
     */
    public function getCartInfo()
    {
        // Sanitize Cart
        $this->returnData = array_intersect_key($this->returnData, array_flip($this->filterHelper->getCheckoutWhitelist()));

        for($i=0;$i<count($this->returnData['items']);$i++)
        {
            $this->returnData['items'][$i] = array_intersect_key($this->returnData['items'][$i], array_flip($this->filterHelper->getItemsWhitelist()));
        }

        return $this->filterHelper->sanitizeAddressData($this->returnData);
    }
}
