<?php

namespace Unific\Extension\Helper;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Historical extends \Magento\Framework\App\Helper\AbstractHelper
{
    protected $historicalItemsInBuffer = 4;

    protected $hmacKey = '';

    protected $subject = 'order/historical';

    protected $logger;
    protected $queueHelper;
    protected $hmacHelper;
    protected $scopeConfig;

    protected $queueCollection;

    protected $searchCriteriaBuilder;

    protected $orderRepository;
    protected $customerRepository;

    protected $categoryFactory;
    protected $productFactory;

    protected $orderDataHelper;
    protected $customerDataHelper;
    protected $cartDataHelper;
    protected $productDataHelper;
    protected $categoryDataHelper;

    protected $writeBuffer = array();

    protected $filterHelper;

    /**
     * ModeManagement constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Extension\Model\ResourceModel\Queue\Collection $queueCollection
     * @param Message\Queue $queueHelper
     * @param Hmac $hmacHelper
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Customer\Model\ResourceModel\CustomerRepository $customerRepository
     * @param \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryFactory
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productFactory
     * @param Filter $filterHelper
     * @param Data\Order $orderDataHelper
     * @param Data\Cart $cartDataHelper
     * @param Data\Customer $customerDataHelper
     * @param Data\Product $productDataHelper
     * @param Data\Category $categoryDataHelper
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Extension\Model\ResourceModel\Queue\Collection $queueCollection,
        Message\Queue $queueHelper,
        Hmac $hmacHelper,
        \Unific\Extension\Logger\Logger $logger,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Sales\Model\OrderRepository $orderRepository,
        \Magento\Customer\Model\ResourceModel\CustomerRepository $customerRepository,
        \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryFactory,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productFactory,
        Filter $filterHelper,
        Data\Order $orderDataHelper,
        Data\Cart $cartDataHelper,
        Data\Customer $customerDataHelper,
        Data\Product $productDataHelper,
        Data\Category $categoryDataHelper
    )
    {
        parent::__construct($context);

        $this->scopeConfig = $context->getScopeConfig();
        $this->logger = $logger;
        $this->queueHelper = $queueHelper;
        $this->hmacHelper = $hmacHelper;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->orderRepository = $orderRepository;
        $this->customerRepository = $customerRepository;
        $this->productFactory = $productFactory;
        $this->categoryFactory = $categoryFactory;
        $this->queueCollection = $queueCollection;
        $this->filterHelper = $filterHelper;

        $this->orderDataHelper = $orderDataHelper;
        $this->customerDataHelper = $customerDataHelper;
        $this->cartDataHelper = $cartDataHelper;
        $this->productDataHelper = $productDataHelper;
        $this->categoryDataHelper = $categoryDataHelper;
    }

    /**
     * Queue up all the historical data to be ready for sending
     */
    public function queueAllHistoricalData()
    {
        $this->hmacKey = $this->scopeConfig->getValue('unific/hmac/hmacHeader', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

        if($this->queueCollection->addFieldToFilter('headers', array('like' => '%/historical%'))->getSize() == 0)
        {
            // Queue Orders
            $this->subject = 'order/historical';
            foreach ($this->orderRepository->getList($this->searchCriteriaBuilder->create()) as $order) {
                $this->orderDataHelper->setOrder($order);
                $this->writeBuffer[] =$this->orderDataHelper->getOrderInfo();
                $this->processWriteBuffer();
            }

            // Force a flush to not get mixed subjects in one historical buffered queue
            $this->processWriteBuffer(true);

            // Queue Customers
            $this->subject = 'customer/historical';
            foreach ($this->customerRepository->getList($this->searchCriteriaBuilder->create())->getItems() as $customer) {
                $this->customerDataHelper->setCustomer($customer);
                $this->writeBuffer[] =$this->customerDataHelper->getCustomerInfo();
                $this->processWriteBuffer();
            }

            // Force a flush to not get mixed subjects in one historical buffered queue
            $this->processWriteBuffer(true);

            // Queue Categories
            $this->subject = 'category/historical';
            $categories = $this->categoryFactory->create()->addAttributeToSelect('*');
            foreach ($categories as $category) {
                $this->categoryDataHelper->setCategory($category);
                $this->writeBuffer[] = $this->categoryDataHelper->getCategoryInfo();
                $this->processWriteBuffer();
            }

            // Force a flush to not get mixed subjects in one historical buffered queue
            $this->processWriteBuffer(true);

            // Queue Products
            $this->subject = 'product/historical';
            $products = $this->productFactory->create()->addAttributeToSelect('*');
            foreach ($products as $product) {
                $this->productDataHelper->setProduct($product);
                $this->writeBuffer[] = $this->productDataHelper->getProductInfo();
                $this->processWriteBuffer();
            }

            // Force a flush to not get mixed subjects in one historical buffered queue
            $this->processWriteBuffer(true);

            return array('message' => 'Historical data queued');
        } else {
            return array('message' => 'Historical data not queued, already processing historical data');
        }
    }

    /**
     * Flush the buffer if its full or forced
     * A buffer will never be sent if its empty
     *
     * @param bool $forceFlush
     */
    public function processWriteBuffer($forceFlush = false)
    {
        // If we have data and its either a full buffer or we forced a flush
        if (count($this->writeBuffer) > 0 && (count($this->writeBuffer) >= $this->historicalItemsInBuffer || $forceFlush)) {
            $extraHeaders = array();

            $extraHeaders['X-SUBJECT'] = $this->subject;
            $extraHeaders[$this->hmacKey] = $this->hmacHelper->generateHmac($this->writeBuffer);

            $this->queueHelper->queue($this->scopeConfig->getValue('unific/extension/endpoint'), $this->writeBuffer, $extraHeaders, \Zend\Http\Request::METHOD_POST, true);

            // Truncate the write buffer
            $this->writeBuffer = array();
        }
    }
}
