<?php

namespace Unific\Extension\Plugin;

class AbstractPlugin
{
    protected $logger;

    protected $restConnection;

    protected $entity = 'order';
    protected $action = 'create';

    protected $dataObjectFactory;

    protected $scopeConfig;

    protected $filterHelper;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        $this->restConnection = $restConnection;
        $this->logger = $logger;

        $this->dataObjectFactory = $dataObjectFactory;

        $this->filterHelper = $filterHelper;
        $this->scopeConfig = $scopeConfig;
    }

    public function processWebhook($data)
    {
        try {
            if (filter_var($this->scopeConfig->getValue('unific/extension/endpoint'), FILTER_VALIDATE_URL) !== FALSE) {
                $response = $this->restConnection->post(
                    $this->scopeConfig->getValue('unific/extension/endpoint'),
                    $data,
                    array(
                        'X-SUBJECT' => $this->getSubject()
                    )
                );

                if (is_string($response)) {
                    $this->logger->info('Message queued for sending: ' . $response);
                } else {
                    $this->logger->info($response->getBody());
                }
            }
        } catch (\Exception $e) {
            $this->logger->error('Exception ' . $e->getCode() . ': ' . $e->getMessage());
        }
    }

    /**
     * @return string
     */
    protected function getSubject()
    {
        return $this->entity . '/' . $this->action;
    }

    /**
     * @param string
     */
    protected function setEntity($entityCode)
    {
        $this->entity = $entityCode;
    }


    /**
     * @param string
     */
    protected function setAction($action)
    {
        $this->action = $action;
    }

    protected function getAction()
    {
        return $this->action;
    }
}