<?php

namespace Unific\Extension\Plugin;

class AdminSessionPlugin extends AbstractPlugin
{
    protected $adminUserDataHelper;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\AdminUser $adminUserDataHelper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\AdminUser $adminUserDataHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->adminUserDataHelper = $adminUserDataHelper;

        $this->setEntity('admin');
    }

    /**
     * @param $subject
     * @return array
     */
    public function afterProcessLogin($subject)
    {
        if($this->scopeConfig->getValue('unific/webhook/admin_login'))
        {
            $this->setAction('login');

            $this->adminUserDataHelper->setUser($subject->getUser());
            $this->processWebhook($this->adminUserDataHelper->getUserInfo());
        }

        return $subject;
    }

    /**
     * @param $subject
     * @return array
     */
    public function beforeProcessLogout($subject)
    {
        if($this->scopeConfig->getValue('unific/webhook/admin_logout'))
        {
            $this->setAction('logout');

            $this->adminUserDataHelper->setUser($subject->getUser());
            $this->processWebhook($this->adminUserDataHelper->getUserInfo());
        }

        return [$subject];
    }
}