<?php

namespace Unific\Extension\Plugin;

class CustomerCartPlugin extends AbstractPlugin
{
    protected $cartDataHelper;
    protected $cartRepository;
    protected $customerSession;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Cart $cartDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Cart $cartDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;
        $this->customerSession = $customerSession;

        $this->setEntity('checkout');
    }

    /**
     * @param $subject
     * @return array
     */
    public function afterSave($subject)
    {
        if($this->customerSession->isLoggedIn())
        {
            if($this->scopeConfig->getValue('unific/webhook/cart')) {

                $this->cartDataHelper->setCart($subject->getQuote());

                if ($this->cartDataHelper->getCart()->getCreatedAt() != $this->cartDataHelper->getCart()->getUpdatedAt() &&
                    $this->cartDataHelper->getCart()->getUpdatedAt() != null) {
                    $this->setAction('update');
                }

                $this->processWebhook($this->cartDataHelper->getCartInfo());
            }
        }

        return $subject;
    }
}
