<?php

namespace Unific\Extension\Plugin;

class CustomerPlugin extends AbstractPlugin
{
    protected $customerDataHelper;

    protected $customerRepository;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Customer $customerDataHelper,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->customerDataHelper = $customerDataHelper;
        $this->customerRepository = $customerRepository;

        $this->setEntity('customer');
    }

    /**
     * @param $subject
     * @param $customer
     * @return mixed
     */
    public function afterSave($subject, $customer)
    {
        if($this->scopeConfig->getValue('unific/webhook/customer')) {
            if ($customer->getCreatedAt() != $customer->getUpdatedAt() && $customer->getUpdatedAt() != null) {
                $this->setAction('update');
            }

            $this->customerDataHelper->setCustomer($customer);
            $this->processWebhook($this->customerDataHelper->getCustomerInfo());
        }

        return $customer;
    }
}
