<?php

namespace Unific\Extension\Plugin;

class CustomerSessionPlugin extends AbstractPlugin
{
    protected $cartDataHelper;
    protected $cartRepository;
    protected $customerSession;
    protected $checkoutSession;
    protected $customerDataHelper;
    protected $customerRepository;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Cart $cartDataHelper
     * @param \Unific\Extension\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Cart $cartDataHelper,
        \Unific\Extension\Helper\Data\Customer $customerDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;
        $this->customerSession = $customerSession;
        $this->checkoutSession = $checkoutSession;
        $this->customerDataHelper = $customerDataHelper;
        $this->customerRepository = $customerRepository;

        $this->setEntity('customer');
    }

    /**
     * @param $subject
     * @param $customer
     * @return array
     */
    public function afterSetCustomerDataAsLoggedIn($subject, $customer)
    {
        if($this->scopeConfig->getValue('unific/webhook/customer_login'))
        {
            $this->setAction('login');
            $this->customerDataHelper->setCustomer($this->customerSession->getCustomer()->getDataModel());
            $this->processWebhook($this->customerDataHelper->getCustomerInfo());
        }

        // Send the cart after logging in
        if($this->customerSession->isLoggedIn() && $this->scopeConfig->getValue('unific/webhook/cart'))
        {
            $this->setEntity('checkout');
            $this->setAction('create');

            $this->cartDataHelper->setCart($this->checkoutSession->getQuote());

            if ($this->cartDataHelper->getCart()->getCreatedAt() != $this->cartDataHelper->getCart()->getUpdatedAt() &&
                $this->cartDataHelper->getCart()->getUpdatedAt() != null) {
                $this->setAction('update');
            }

            $this->processWebhook($this->cartDataHelper->getCartInfo());
        }

        return $customer;
    }

    /**
     * @param $subject
     * @return array
     */
    public function beforeLogout($subject)
    {
        if($this->scopeConfig->getValue('unific/webhook/customer_logout'))
        {
            $this->setAction('logout');
            $this->customerDataHelper->setCustomer($this->customerSession->getCustomer()->getDataModel());
            $this->processWebhook($this->customerDataHelper->getCustomerInfo());
        }

        return [$subject];
    }
}
