<?php

namespace Unific\Extension\Plugin;

class OrderPlugin extends AbstractPlugin
{
    protected $orderDataHelper;

    protected $cartDataHelper;
    protected $cartRepository;

    protected $customerSession;
    protected $customerDataHelper;

    protected $orderAction = 'create';

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Order $orderDataHelper
     * @param \Unific\Extension\Helper\Data\Cart $cartDataHelper
     * @param \Unific\Extension\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Order $orderDataHelper,
        \Unific\Extension\Helper\Data\Cart $cartDataHelper,
        \Unific\Extension\Helper\Data\Customer $customerDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;

        $this->orderDataHelper = $orderDataHelper;
        $this->customerSession = $customerSession;
        $this->customerDataHelper = $customerDataHelper;

        $this->setEntity('order');
    }

    /**
     * @param \Magento\Sales\Api\OrderRepositoryInterface $subject
     * @param \Magento\Sales\Api\Data\OrderInterface $entity
     * @return \Magento\Sales\Api\Data\OrderInterface
     */
    public function afterSave($subject,\Magento\Sales\Api\Data\OrderInterface $entity)
    {
        if($this->scopeConfig->getValue('unific/webhook/order')) {
            switch ($entity->getState()) {
                case \Magento\Sales\Model\Order::STATE_PROCESSING:
                    $this->setAction('update');
                    break;

                case \Magento\Sales\Model\Order::STATE_CANCELED:
                    $this->setAction('cancel');
                    break;

                case \Magento\Sales\Model\Order::STATE_COMPLETE:
                    $this->setAction('ship');
                    break;

                case \Magento\Sales\Model\Order::STATE_CLOSED:
                    $this->setAction('refund');
                    break;
            }

            $this->orderAction = $this->getAction();

            // Send the order
            $this->orderDataHelper->setOrder($entity);
            $this->processWebhook($this->orderDataHelper->getOrderInfo());
        }

        // Send the cart
        if($this->orderAction == 'create' && $this->scopeConfig->getValue('unific/webhook/cart')) {
            $this->setEntity('checkout');
            $this->setAction('update');

            $this->cartDataHelper->setCart($this->cartRepository->get($entity->getQuoteId()));
            $this->processWebhook($this->cartDataHelper->getCartInfo());
        }

        // Send customer
        if($this->orderAction == 'create' && $this->scopeConfig->getValue('unific/webhook/customer')) {
            $this->setEntity('customer');
            $this->setAction('update');

            // Send a customer update, if not logged in, generate a customer from order data
            if ($this->customerSession->isLoggedIn()) {
                $this->customerDataHelper->setCustomer($this->customerSession->getCustomer()->getDataModel());
            } else {
                $this->customerDataHelper->generateGuestCustomer($entity);
            }

            $this->processWebhook($this->customerDataHelper->getCustomerInfo());
        }

        return $entity;
    }
}
