<?php

namespace Unific\Extension\Plugin;

class ProductPlugin extends AbstractPlugin
{
    protected $productDataHelper;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Product $productDataHelper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Product $productDataHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->productDataHelper = $productDataHelper;
    }

    /**
     * @param \Magento\Catalog\Model\Product $subject
     * @return \Magento\Catalog\Model\Product
     */
    public function afterSave(\Magento\Catalog\Model\Product $subject)
    {
        if($this->scopeConfig->getValue('unific/webhook/product')) {
            $this->setEntity('product');

            if($subject->getCreatedAt() != $subject->getUpdatedAt() && $subject->getUpdatedAt() != null)
            {
                $this->setAction('update');
            }

            $this->productDataHelper->setProduct($subject);
            $this->processWebhook($this->productDataHelper->getProductInfo());
        }

        return $subject;
    }
}
